#ifndef __WMP_H
#define __WMP_H

#include "types.h"

extern unsigned int wmp_yaw;
extern unsigned int wmp_pitch;
extern unsigned int wmp_roll;

extern unsigned char wmp_calibration_data[];

extern bool wmp_yaw_fast;
extern bool wmp_pitch_fast;
extern bool wmp_roll_fast;

bool wmp_init(void);
bool wmp_sample(void);
bool wmp_read_calibration_data(void);
bool wmp_start_sample(void);
void wmp_event_handler(void);

#endif /* __WMP_H */
